/// <reference types="node" />
import * as frontend from 'llparse-frontend';
import source = frontend.source;
import * as cImpl from '../implementation/c';
import * as jsImpl from '../implementation/js';
export interface ICompilerOptions {
    /**
     * Debug method name
     *
     * The method must have following signature:
     *
     * ```c
     * void debug(llparse_t* state, const char* p, const char* endp,
     *            const char* msg);
     * ```
     *
     * Where `llparse_t` is a parser state type.
     */
    readonly debug?: string;
    /**
     * What guard define to use in `#ifndef` in C headers.
     *
     * Default value: `prefix` argument
     */
    readonly headerGuard?: string;
    /** Generate bitcode (`true` by default) */
    readonly generateBitcode?: boolean;
    /** Generate C (`true` by default) */
    readonly generateC?: boolean;
    /** Generate JS (`true` by default) */
    readonly generateJS?: boolean;
    /** Optional frontend configuration */
    readonly frontend?: frontend.IFrontendLazyOptions;
    /** Optional C-backend configuration */
    readonly c?: cImpl.ICPublicOptions;
    /** Optional JS-backend configuration */
    readonly js?: jsImpl.IJSPublicOptions;
}
export interface ICompilerResult {
    /**
     * Binary LLVM bitcode, if `generateBitcode` option was `true`
     */
    readonly bitcode?: Buffer;
    /**
     * Textual C code, if `generateC` option was `true`
     */
    readonly c?: string;
    /**
     * Textual JS code, if `generateJS` option was `true`
     */
    readonly js?: string;
    /**
     * Textual C header file
     */
    readonly header: string;
}
export declare class Compiler {
    readonly prefix: string;
    readonly options: ICompilerOptions;
    constructor(prefix: string, options: ICompilerOptions);
    compile(root: source.node.Node, properties: ReadonlyArray<source.Property>): ICompilerResult;
}
