/// <reference types="node" />
import * as frontend from 'llparse-frontend';
import { Code } from './code';
import { Node } from './node';
import { Transform } from './transform';
declare type WrappedNode = frontend.IWrap<frontend.node.Node>;
export interface ICompilationOptions {
    readonly debug?: string;
    readonly module: 'esm' | 'commonjs';
}
export interface ICompilationProperty {
    readonly name: string;
    readonly ty: string;
}
export declare class Compilation {
    readonly prefix: string;
    private readonly properties;
    private readonly options;
    private readonly stateMap;
    private readonly blobs;
    private readonly tables;
    private readonly codeMap;
    private readonly matchSequence;
    constructor(prefix: string, properties: ReadonlyArray<ICompilationProperty>, options: ICompilationOptions);
    private buildStateEnum;
    private buildBlobs;
    private buildTables;
    private buildMatchSequence;
    reserveSpans(spans: ReadonlyArray<frontend.SpanField>): void;
    debug(out: string[], message: string): void;
    buildGlobals(out: string[]): void;
    buildMethods(out: string[]): void;
    buildStates(out: string[]): void;
    addState(state: string, lines: ReadonlyArray<string>): void;
    buildCode(code: Code<frontend.code.Code>): string;
    getFieldType(field: string): string;
    exportDefault(lines: string[]): string;
    importCode(name: string, out: string[]): void;
    unwrapCode(code: frontend.IWrap<frontend.code.Code>): Code<frontend.code.Code>;
    unwrapNode(node: WrappedNode): Node<frontend.node.Node>;
    unwrapTransform(node: frontend.IWrap<frontend.transform.Transform>): Transform<frontend.transform.Transform>;
    indent(out: string[], lines: ReadonlyArray<string>, pad: string): void;
    getMatchSequence(transform: frontend.IWrap<frontend.transform.Transform>, select: Buffer): string;
    bufArg(): string;
    offArg(): string;
    currentArg(): string;
    stateVar(): string;
    matchVar(): string;
    indexField(): string;
    currentField(): string;
    statusField(): string;
    errorField(): string;
    reasonField(): string;
    errorOffField(): string;
    spanOffField(index: number): string;
    spanCbField(index: number): string;
    stateField(name: string): string;
    blob(value: Buffer): string;
    table(value: ReadonlyArray<number>): string;
}
export {};
