import * as frontend from 'llparse-frontend';

import { Compilation } from '../compilation';
import { Node } from './base';

export class Single extends Node<frontend.node.Single> {
  public doBuild(out: string[]): void {
    const ctx = this.compilation;

    const otherwise = this.ref.otherwise!;

    this.prologue(out);

    const transform = ctx.unwrapTransform(this.ref.transform!);
    const current = transform.build(ctx, `${ctx.bufArg()}[${ctx.offArg()}]`);

    out.push(`switch (${current} | 0) {`)
    this.ref.edges.forEach((edge) => {
      let ch: string = edge.key.toString();

      // Non-printable ASCII, or single-quote
      if (!(edge.key < 0x20 || edge.key > 0x7e || edge.key === 0x27)) {
        ch = `/* '${String.fromCharCode(edge.key)}' */ ` + ch;
      }
      out.push(`  case ${ch}: {`);

      const tmp: string[] = [];
      this.tailTo(tmp, edge);
      ctx.indent(out, tmp, '    ');

      out.push('  }');
    });

    out.push(`  default: {`);

    const tmp: string[] = [];
    this.tailTo(tmp, otherwise);
    ctx.indent(out, tmp, '    ');

    out.push('  }');

    out.push(`}`);
  }
}
